/***************************************************************************
                        rl3964r.cpp  -  description
                             -------------------
    begin                : Wed Jan 14 2004
    copyright            : (C) 2004 by R. Lehrig
    email                : lehrig@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU LESSER GENERAL PUBLIC LICENSE as        *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "rlcutil.h"
#include "rl3964r.h"
#define STX 2
#define ETX 3
#define ENQ 5
#define EOT 4
#define ETB 23
#define DLE 16
#define NAK 21
#define ACK 6

enum State
{
  IDLE            = 0,
  SENDING         = 1,
  RECEIVING       = 2,
  WANT_TO_SEND    = 3,
  WANT_TO_RECEIVE = 4
};

static const char *statestr[] = {"idle","sending","receiving","want_to_send","want_to_receive"};

static void *receiverThread(void *arg)
{
  THREAD_PARAM *p = (THREAD_PARAM *) arg;
  rl3964R      *d = (rl3964R *) p->user;
  unsigned char c;
  int ret,send_retry;

  send_retry = 0;
  while(d->run)
  {
    c = 0x0ff;
    ret = d->tty.select(1000);
    if(ret == 1) c = d->tty.readChar();
    p->thread->lock();

    d->dprintf("receiverThread: c=%d c=%x state=%s\n",c,c,statestr[d->state]);
    if(c==STX && d->state==IDLE)
    {
      d->dprintf("STX IDLE\n");
      d->state = RECEIVING;
      d->dprintf("write DLE\n");
      d->tty.writeChar(DLE);
      d->dprintf("call receive\n");
      d->receive();
      d->dprintf("after receive\n");
      d->state = IDLE;
    }
    else if(c==STX && d->state==WANT_TO_SEND && d->priority==rl3964R::lowPriority)
    {
      d->dprintf("receiverThread: STX WANT_TO_SEND lowPriority\n");
      d->state = RECEIVING;
      d->tty.writeChar(DLE);
      d->receive();
      d->state = WANT_TO_SEND;
    }
    else if(c==STX && d->state==WANT_TO_SEND && d->priority==rl3964R::highPriority)
    {
      d->dprintf("receiverThread: STX WANT_TO_SEND highPriority\n");
      d->tty.writeChar(NAK);
      d->tty.writeChar(STX);
    }
    else if(c==DLE && d->state==WANT_TO_SEND)
    {
      d->dprintf("receiverThread: DLE WANT_TO_SEND\n");
      d->send();
      d->state = IDLE;
      d->receiver.incrementSemaphore();
    }
    else if(c==NAK)
    {
      d->dprintf("receiverThread: NAK\n");
    }
    else if(d->state==WANT_TO_SEND)
    {
      if(send_retry < 1)
      {
      }
      else if(send_retry < 3)
      {
        d->tty.writeChar(STX);
        send_retry++;
        d->dprintf("receiverThread: WANT_TO_SEND send=STX retry=%d\n",send_retry+1);
      } 
      else
      {
        send_retry = 0;
        d->dprintf("receiverThread: WANT_TO_SEND failed after 3 retries\n");
        d->state = IDLE;
        d->receiver.incrementSemaphore();
      }
    }
    else
    {
      d->dprintf("receiverThread: IDLE\n");
      d->state = IDLE;
    }

    p->thread->unlock();
  }

  return NULL;
}

rl3964R::rl3964R(int _priority)
{
  priority = _priority;
  readCallback = NULL;
  debug = 0;
  isOpen = 0;
  state = IDLE;
  send_result = 0;
  tel_receive_length = tel_send_length = -1;
}

rl3964R::~rl3964R()
{
  close();
}

int rl3964R::open(const char *devicename, int _baudrate)
{
  int ret;

  if(isOpen == 0)
  {
    //int openDevice(const char *devicename, int speed=B9600, int block=1, int rtscts=1, int bits=8, int stopbits=1, int parity=NONE);
    ret = tty.openDevice(devicename,_baudrate,1,0,8,1,rlSerial::EVEN);
    if(ret >= 0)
    {
      isOpen = 1;
      run = 1;
      receiver.create(receiverThread,this);
    }
    return ret;
  }
  return -1;
}

int rl3964R::close()
{
  if(isOpen == 1)
  {
    isOpen = 0;
    run = 0;
    dprintf("close(): cancel\n");
    receiver.cancel();
    dprintf("close(): closeDevice\n");
    tty.closeDevice();
    dprintf("close(): return\n");
    return 0;
  }
  isOpen = 0;
  return -1;
}

int rl3964R::setReadCallback( void (*_readCallback)(const unsigned char *buf, int len))
{
  readCallback = _readCallback;
  return 0;
}

int rl3964R::write(const unsigned char *buf, int len)
{
//  tty.setTrace(1);
  dprintf("write() len=%d\n",len);
  if(len >= (int) sizeof(tel_send)) return -1;
  receiver.lock();
  tel_send_length = len;
  memcpy(tel_send,buf,len);
  state = WANT_TO_SEND;
  dprintf("write() STX\n");
  tty.writeChar(STX);
  dprintf("write() unlock\n");
  receiver.unlock();
  dprintf("write() waitSemaphore\n");
  receiver.waitSemaphore();
  dprintf("write() return len=%d\n",len);
  if(send_result < 0) return -1;
  return len;
}

int rl3964R::send()
{
  int i,bcc,c,ret,length_part,teiler_laenge;
  
  bcc = 0;
  tel_receive[0]=tel_send[2]; /* Auftrag: 0x41 = Daten anfordern 0x45 = Daten schicken*/
  tel_receive[1]=tel_send[3]; /* Typ: 0x44 = DB-DW; 0x4D = Merker; 0x58 = DX-DW siehe case : */
  tel_receive[2]=tel_send[6]; /* Angeforderte Datenlänge Teil 1*/
  tel_receive[3]=tel_send[7]; /* Angeforderte Datenlänge Teil 2*/
  
  switch(tel_receive[1])
  {
  	case 'D': //DB Für Word 
		teiler_laenge = 2;
		break;
	case 'X': //DX Für Word 
		teiler_laenge = 2;
		break;
	case 'Z': //????? Zählerwerte Für Word 
		teiler_laenge = 2;
		break;
	case 'T': //????? Timerwerte Für Word 
		teiler_laenge = 2;
		break;
	// Hier müssen noch die anderen Formate getestet werden	
  	default: //Für Byte
		teiler_laenge = 1;
		break;
  }
  length_part=(tel_receive[2] * 256 + tel_receive[3]) * teiler_laenge + 10;
  if(tel_receive[0]==0x45) length_part = 10;
  for(i=0; i<length_part; i++)
  {
    switch(tel_send[i])
    {
      case DLE:
        tty.writeChar(DLE);
        tty.writeChar(DLE);
        bcc = bcc ^ DLE;
        bcc = bcc ^ DLE;
  	dprintf("send() sends 2xDLE\n");
        break;
      default:
        tty.writeChar(tel_send[i]);
        bcc = bcc ^ tel_send[i];
  	dprintf("send()sends %2x\n",tel_send[i]);
        break;
    }
  }
  tty.writeChar(DLE);
  dprintf("send()sends %2x DLE\n",DLE);
  bcc = bcc ^ DLE;
  tty.writeChar(ETX);
  dprintf("send()sends %2x ETX\n",ETX);
  bcc = bcc ^ ETX;
  tty.writeChar(bcc);
  dprintf("send()sends %2x BCC\n",bcc);
  ret = tty.select(1000);
  dprintf("send()tty.select %2x\n",ret);
  if(ret == 1) 
  {
    c = tty.readChar();
    dprintf("send()tty.readchar %2x\n",c);
  }
  else
  {
    send_result = -1;
    dprintf("send(): partner was sending nothing\n");
    return -1;
  }  
  if(c == DLE)
  {
    send_result = 0;
    dprintf("send() success\n");
    return tel_send_length;
  }
  send_result = -1;
  dprintf("send() failure\n");
  return -1;
}

int rl3964R::receive()
{
  int i,c,c2,bcc,received_bcc,ret,ordertyp,byte_word,teiler_laenge,length_part;
  
  ordertyp=tel_receive[0];
  byte_word=tel_receive[1];
  switch(byte_word)
  {
	case 'D': //DB Für Word 
		teiler_laenge = 2;
		break;
	case 'X': //DX Für Word 
		teiler_laenge = 2;
		break;
	case 'Z': //????? Zählerwerte Für Word 
		teiler_laenge = 2;
		break;
	case 'T': //????? Timerwerte Für Word 
		teiler_laenge = 2;
		break;
	// Hier müssen noch die anderen Formate getestet werden	
  	default: //Für Byte
		teiler_laenge = 1;
		break;
  }
  length_part=(tel_receive[2] * 256 + tel_receive[3]) * teiler_laenge + 7 - 1;
  if(ordertyp==0x41) length_part = 7 - 1;
  dprintf("receive() Ordertyp=%x length_part=%d\n",ordertyp,length_part);
  //bcc = STX;
  bcc = 0;
//  c = STX ;
  i = received_bcc = 0;
//  tel_receive[i++] = c =  STX;
//  while(c > 0 && i < (int) sizeof(tel_receive))
  while(i < length_part)
  {
    ret = tty.select(1000);
    if(ret == 1) c = tty.readChar();
    else
    {
      dprintf("receive(): partner was sending nothing\n");
      return -1;
    }  
//    dprintf("receive()switch(c= %d) ret=%d tel_receive=%d\n",c,i,(int) sizeof(tel_receive));
    switch(c)
    {
      case -1:
      case -2:
        return -1;
      case DLE:
        bcc = bcc ^ c;
        dprintf("receive() case DLE %x counter: %d\n",c,i);
        c2 = tty.readChar();
        bcc = bcc ^ c2;
        if(c2 < 0) return -1;
        tel_receive[i++] = c2;
        if(c2 == ETX)
        {
          dprintf("receive() ETX %x counter: %d\n",c2,i);
          c2 = tty.readChar();
          if(c2 < 0) return -1;
          tel_receive[i++] = c2; // bcc
          received_bcc = c2;
          dprintf("receive() bcc=%d received_bcc=%d\n",bcc,received_bcc);
          c = -1;         
        }
        break;
      default:
        bcc = bcc ^ c;
        tel_receive[i++] = c;
    	dprintf("receive()switch(c=%x) counter: %d\n",c,i);
        break;
    }
  }
  tel_receive_length = i;

  if(bcc == received_bcc)
  {
    tty.writeChar(DLE);
    dprintf("receive() success DLE not necessesary!! bcc=%d received_bcc=%d\n",bcc,received_bcc);
//    if(readCallback != NULL) (readCallback)(&tel_receive[1],tel_receive_length-3);
    if(readCallback != NULL) (readCallback)(&tel_receive[0],tel_receive_length-0);
    return tel_receive_length-0;
  }
  tty.writeChar(NAK);
  dprintf("receive() failure\n");
  return -1;
}

int rl3964R::dprintf(const char *format, ...)
{
  char message[rl_PRINTF_LENGTH]; // should be big enough
  int ret;

  if(debug != 1) return 0;
  va_list ap;
  va_start(ap,format);
  ret = rlvsnprintf(message, rl_PRINTF_LENGTH - 1, format, ap);
  va_end(ap);
  printf("%s",message);
  return ret;
}

