/* <St> *******************************************************************

  FILENAME    : COM_USER.H

  -------------------------------------------------------------------------
  CREATED BY  : Armin Beck,  Hilscher GmbH
  CREATED AT  : 23.06.97
  PROJECT     :
  =========================================================================

  FUNCTION :
    User-Interface to CANopen-master

  =========================================================================

  CHANGES OF REVISIONS :

  Version Name      Date      Change
  -------------------------------------------------------------------------
  V1.026  AB        30.05.01  - definition of cplusplus directive
  V1.025  AB        24.01.01  - structure COM_ND_PRM_HEADER extended by
                                bitCfgNdState variable
  V1.024  AB        12.01.00  - COM_Send_CANMsg_Transparent included
  V1.023  AB        11.01.00  - new error counter in structure COM_DIAGNOSTICS
                                named usRx_OverFlow
  V1.022  AB        07.07.99  - rename CANM_ to COM_
  V1.021  AB        05.01.99  - extention of structure CANM_PLC_PARAMETER
                                in usWatchDogTime-Variable
  V1.020  AB        08.08.98  - CANM_SDO_Up_Download-command inserted
  V1.011  AB        27.11.97  - TOUT-Bit in the global bus state structure
                                inserted
  V1.010  AB        11.11.97  - adjusted to general header file definitions
  V1.001  AB        10.09.97  - extended bus parameters, 'bIntelMotorola'
                              - new funtionality of PDO size indicator
                                'bLength'
                              - Offset for PDO now byte offset no longer
                                word offset
                              - extended error numbers
                              - CAN message timeout counter in DIAGNOSTIC
                                structure
                              - replace definition CAN_ with COM_
  V1.000  AB        23.06.96  - created

  ******************************************************************** <En> */

#ifndef __COM_USER_H
  #define __COM_USER_H

#if _MSC_VER >= 1000
#pragma once
#endif /* _MSC_VER >= 1000 */

#ifdef __cplusplus
  extern "C" {
#endif

#if defined( _MSC_VER)          /* Microsoft C */
    #pragma pack(1)             /* Byte Alignment   */
#endif

/* ------------------------------------------------------------------------------------ */
/*   process image transfer mode definitions                                            */
/* ------------------------------------------------------------------------------------ */

#define COM_GET_MODE_MASK                       0xf0

#define COM_GET_MODE_NOT_SUPPORTED              0x00
#define COM_GET_MODE_BUSSYNC_DEVICE_CONTROLLED  0x10
#define COM_GET_MODE_BUFFERED_DEVICE_CONTROLLED 0x20
#define COM_GET_MODE_UNCONTROLLED               0x30
#define COM_GET_MODE_BUFFERED_HOST_CONTROLLED   0x40
#define COM_GET_MODE_BUSSYNC_HOST_CONTROLLED    0x50


/* ======================================================================== */
/* Protocol parameter structure                                             */
/* ======================================================================== */

/* --------------------------- */
/* device warm start parameter */
/* --------------------------- */

typedef struct COM_PLC_PARAMETERtag {
  unsigned char  bMode;         /* handshake of process data */
  #define COM_SET_MODE_BUFFERED_DEVICE_CONTROLLED 1
  #define COM_SET_MODE_UNCONTROLLED               2
  #define COM_SET_MODE_BUFFERED_HOST_CONTROLLED   3
  unsigned short usWatchDogTime;
  unsigned char  abReservedA[5];
  unsigned char  abReservedB[8];
} COM_PLC_PARAMETER;

/* ------------------------------------------------------------- */
/* structures to set node data set by message download procedure */
/* ------------------------------------------------------------- */

/* parameter data header structure of a node station */

typedef struct COM_ND_PRM_HEADERtag {
  unsigned short  usNodeParaLen;  /* length of whole parameter data set+2 */
  struct
  {
    unsigned char bReserved    : 7;
    unsigned char bActive      : 1;
  } bNdFlag;
  unsigned short  usAddTypeInfo;    /* corresponding module type OBJ1000H */
  unsigned short  usDevProfNum;     /* corresponding profile number OBJ1000H*/
  unsigned short  usCobIDEmcy;      /* EMCY-COB-ID */
  unsigned short  usCobIDGuard;     /* NDGRD-COB-ID */
  unsigned short  usGuardTime;      /* guard-time in multiples of 1msec */
  unsigned char   bLifeTimeFactor;        /* multiplcator for the guard-time */

  struct
  {
    unsigned biNoNodeReset: 1;
    unsigned biNoCmpDevice: 1;
    unsigned biNoCfgGuard : 1;
    unsigned biNoSyncId   : 1;
    unsigned biNoEmcyId   : 1;
    unsigned biNoCfgSdo   : 1;
    unsigned biNoStartNode: 1;
    unsigned biRTxRxReq   : 1;
  } bitCfgNdState;

  unsigned char   abReserved[1];
} ;

/* SDO-parameter data structure of a node station */

#define  MAX_SDO_DATA_SIZE 4
typedef struct COM_SDO_CFG_DATAtag {
  unsigned short  usIndex;         /* index of the SDO to download */
  unsigned char   bSubIndex;       /* subindex of the SDO to download */
  unsigned short  usLength;        /* length of the data */
  unsigned char   abSdoValue[MAX_SDO_DATA_SIZE];
} COM_SDO_CFG_DATA;

#define  MAX_SDOS  100
typedef struct  COM_ND_SDO_CFG_DATAtag {
  unsigned short    usSdoCfgDataLen; /* length of the SDO cfg. data field +2*/
  COM_SDO_CFG_DATA atSdoCfgData[MAX_SDOS]; /* SDO cfg. dat field */
} COM_ND_SDO_CFG_DATA;

/* PDO-configuration data structure of a node station */

typedef struct COM_PDO_DATAtag {
  unsigned char   bTypOfPdo;  /* type of PDO */
  #define RXPDO 0             /* RX-PDO = outputs in the view of the master */
  #define TXPDO 1             /* TX-PDO = inputs in the view of the master */
  unsigned char   bLength;    /* length of valid data in PDO */
  #define MSK_LENGTH   0x7f   /* mask for real length */
  #define MSK_WORD_PDO 0x80   /* bit-mask to detect word oriented PDO */
  unsigned short  usPdoCOBId; /* COB-ID of PDO */
  unsigned char   bTransType; /* transm. type as CiA Draft Standard 301 */
  unsigned short  usInhibitCycle;  /* inhibit time in multiples of node handler cycles */
                                  /* if 0 then event driven by changing data */
} COM_PDO_DATA;

#define MAX_PDOS 10
typedef struct COM_ND_PDO_CFG_DATAtag {
  unsigned short  usPdoCfgDataLen; /* length of the PDO-cfg. data field+2*/
  COM_PDO_DATA    atPdoCfgData[MAX_PDOS]; /* PDO-cfg data field */
} COM_ND_PDO_CFG_DATA;

/* i/o offset table structure of a node station */

typedef struct COM_ND_PRM_ADD_TABtag {
  unsigned short  usAddTabLen; /* length of ADD-tab data field */
  unsigned char   bInputCount;  /* counter of following input offsets */
  unsigned char   bOutputCount; /* counter of following output offsets */
  unsigned short  ausIO_Offset[MAX_PDOS]; /* user defined byte-offsets */
                                          /* first all inputs then all */
                                          /* output offsets            */
} COM_ND_PRM_ADD_TAB;

/* ------------------------------------------------------------------- */
/* structure to set master bus parameter by message download procedure */
/* ------------------------------------------------------------------- */

typedef struct BUS_CANtag {
  unsigned char bBaudrate;    /* baudrate of CAN system */
  #define COM_BAUD_10   8
  #define COM_BAUD_20   7
  #define COM_BAUD_50   6
  #define COM_BAUD_100  5
  #define COM_BAUD_125  4
  #define COM_BAUD_250  3
  #define COM_BAUD_500  2
  #define COM_BAUD_800  1
  #define COM_BAUD_1000 0
  unsigned short usCobIdSync;  /* COB-ID for SYNC-message 1-2048 */
  unsigned short usCobIdReloadTime; /* timer for SYNC in msec 1-65535 */
  unsigned char  bAutoClear;   /* stop transmission if one node is faulty */
  #define ACLR_INACTIVE 0
  #define ACLR_ACTIVE   1
  unsigned char  bFormat;   /* int/mot.form. for word oriented PDO data */
  #define        COM_FORMAT_INTEL   0
  #define        COM_FORMAT_MOTROLA 1
} BUS_CAN;

typedef struct BUS_CAN_EXTtag {
  struct
  {
    unsigned bi11Bit29BitSelector: 1;
    unsigned biNoGobalStartNode  : 1;
    unsigned biReserved          : 6;
  } bExtendedState;
  unsigned long  ulAcceptCode;
  unsigned long  ulAcceptMask;
  unsigned long  ulReserved[8];
} BUS_CAN_EXT;


/* ======================================================================== */
/* Protocol node diagnostic structure read out by message                   */
/* ======================================================================== */

typedef struct COM_EMCY_ENTRYtag{
  unsigned short usEmcyErrCode;
  unsigned char  usEmcyErrorReg_OBJ1001;
  unsigned long  abManuSpecErrField_OBJ1002;
  unsigned char  bReserved;
} COM_EMCY_ENTRY;

#define MAX_ERROR_PER_NODE 5
typedef struct COM_ND_DIAGNOTICStag {
  struct
  {
    unsigned char    bNodeNoResponse   : 1; /* no response */
    unsigned char    bEmcyBuffOverflow : 1; /* emcy.buffer overflow */
    unsigned char    bPrmFault         : 1; /* parameters faulty */
    unsigned char    bGuardActive      : 1; /* node guarding active */
    unsigned char    bReserved         : 3;
    unsigned char    bDeactivated      : 1; /* not configured */
  } bNodeStatus_1;
  unsigned short usAddInfo;          /* read out from OBJ1000H */
  unsigned short usProfileNumber;    /* read out from OBJ1000H */
  unsigned char  bNodeState;         /* state of node  */
  #define DISCONNECTED       1
  #define CONNECTING         2
  #define PREPARING          3
  #define PREPARED           4
  #define OPERATIONAL        5
  #define PRE_OPERATIONAL  127
  unsigned char  bActualError;       /* actual online error value */
  unsigned char  bEmcyEntries;       /* number of saved EMCY-messages */
  COM_EMCY_ENTRY atEmcyData [MAX_ERROR_PER_NODE];
} COM_ND_DIAGNOSTICS;


/* ======================================================================== */
/* Protocol Master state structure                                          */
/* ======================================================================== */

typedef struct COM_DIAGNOSTICStag {
  /* bit field to show bus and master main errors */
  struct
  {
    unsigned char bCtrl     : 1; /* wrong parameterization                  */
    unsigned char bAClr     : 1; /* auto_clear activated, bus stopped       */
    unsigned char bNonExch  : 1; /* no data exchange to on node             */
    unsigned char bFatal    : 1; /* fatal error occured                     */
    unsigned char bEvent    : 1; /* bus error events occured                */
    unsigned char bNRdy     : 1; /* host program not ready                  */
    unsigned char bTOut     : 1; /* CAN telegrams rejected because of timeo.*/
    unsigned char bReserved : 1;
  } bGlobalBits;

  /* global state for the different master main states */
  unsigned char   bCAN_state;
  #define OFFLINE  0x00
  #define STOP     0x40
  #define CLEAR    0x80
  #define OPERATE  0xC0

  /* location of error and error code */
  struct
  {
    unsigned char bErr_Node_Adr; /* 0-126, 255 */
    #define MST_ERR  0xFF
    unsigned char bErr_Event;    /* see #defines */

    #define TASK_F_UNKNOWN_MODE      52  /* unknown handshake mode */
    #define TASK_F_ND_ADR_DOUBLE     60  /* double node address projected */
    #define TASK_F_WATCHDOG         220  /* HOST-Watchdog error */
    #define TASK_F_AUTO_CLEAR       222  /* auto clear activated */

    #define COM_GUARDING_FAILED     30  /* guarding failed for node */
    #define COM_GUARDING_STATE      31  /* wrong state in guarding message */
    #define COM_GUARDING_TOGGLE     32  /* wrong toggle bit received */
    #define COM_RTR_TIMEOUT         33  /* timeout in PDO-RTR acknowledge */
    #define COM_SDO_TIME_OUT        34  /* timeout while SDO access */
    #define COM_PROFILE_FAILURE     35  /* wrong profile number */
    #define COM_DEV_TYPE_FAILURE    36  /* wrong add type info */
  } tError;

  /* counter for the bus error events */
  unsigned short  usBus_Error_Cnt;

  /* counter for the bus off events */
  unsigned short  usBus_Off_Cnt;

  /* counter for the CAN-Message timeouts */
  unsigned short  usMsg_Time_out;

  /* counter for indicated Receive Message Overflows */
  unsigned short  usRx_OverFlow;

  /* reserved area */
  unsigned char   abReserved[4];

  /* Bit-Ready,Prm and diagnostic display of node devices */
  unsigned char   abNd_cfg  [16];
  unsigned char   abNd_state[16];
  unsigned char   abNd_diag [16];
} COM_DIAGNOSTICS;

/* ======================================================================== */
/* Protocol definition                                                      */
/* ======================================================================== */

/* ------------------------------- */
/* message: Send_Transparent       */
/* ------------------------------- */


#define COM_Send_CANMsg_Transparent 0x41

typedef struct COM_SEND_CANMSG_TRANSPARENTtag {
  unsigned char bCANHigh;
  unsigned char bCANLow;
  unsigned char abCANData[8];
} COM_SEND_CANMSG_TRANSPARENT;

/* ------------------------------- */
/* message: shared_memory          */
/* ------------------------------- */

/* message command msg.b msg.a */
#define COM_Shared_Memory   0x11


/* ------------------------------- */
/* message: slave_diag             */
/* ------------------------------- */

/* message command msg.b msg.a */
#define COM_Node_Diag       0x42

typedef struct COM_NODE_DIAG_CONFIRMtag {
  COM_ND_DIAGNOSTICS tDiagData;
} COM_NODE_DIAG_CONFIRM;


/* ------------------------------- */
/* message: start_seq              */
/* ------------------------------- */

/* message command msg.b msg.a */
#define COM_Start_Seq       0x43

typedef struct COM_START_SEQ_REQUESTtag {
   unsigned char        bReq_Add;   /* 0 */
   unsigned char        bArea_Code; /* 0 - 126 */
   unsigned short       usTimeout;  /* 0 - 65535 */
} COM_START_SEQ_REQUEST;

typedef struct COM_START_SEQ_CONFIRMtag {
   unsigned char        bMax_Len_Data_Unit;   /* 240 */
} COM_START_SEQ_CONFIRM;


/* ------------------------------- */
/* message: download               */
/* ------------------------------- */

#define MAX_LEN_DATA_UNIT    240

/* message command msg.b msg.a */
#define COM_Download        0x44

/* message data structure */
typedef struct COM_DOWNLOAD_REQUESTtag {
   unsigned char        bReq_Add;    /* 0 */
   unsigned char        bArea_Code;  /* 0 - 126, 127 */
   unsigned short       usAdd_Offset; /* 0 - xxx */
   unsigned char        abData[MAX_LEN_DATA_UNIT];
} COM_DOWNLOAD_REQUEST;


/* ------------------------------- */
/* message: end_sequence           */
/* ------------------------------- */

/* message command msg.b msg.a */
#define COM_End_Seq         0x45

/* message data structure */
typedef struct COM_END_SEQ_REQUESTtag {
   unsigned char        bReq_Add;   /* 0 */
} COM_END_SEQ_REQUEST;


/* ------------------------------- */
/* message: SDO_Up_Download        */
/* ------------------------------- */

/* message command msg.b msg.a */
#define COM_SDO_Up_Download  0x4A


/* ======================================================================== */
/* message error numbers                                                    */
/* ======================================================================== */

#define  CON_OK         0x00    /* no error                                 */
#define  CON_IV         0x15    /* parameter fault in request               */
#define  CON_TO         0x30    /* function timeout                         */
#define  CON_NI         0x34    /* function not available on responder      */
#define  CON_EA         0x35    /* area overflow          (up-/download)    */
#define  CON_IP         0x37    /* parameter error                          */
#define  CON_SE         0x39    /* sequence fault                           */
#define  CON_DI         0x3B    /* data fault or not complete               */


#define  COM_NODE_ADR_DOUBLE    60 /* Node address configured twice */
#define  COM_ADD_TAB_DATA_SIZE  61 /* ADD-tab size indicator too large */
#define  COM_DATA_SET_SIZE      62 /* the whole node prm. data size too large */
#define  COM_UNKNOWN_TRANS_TYPE 63 /* unknown transmission type */
#define  COM_CONFIG_DATA_SIZE   64 /* PDO-cfg data size indicator too large */
#define  COM_BAUDRATE_FAILURE   65 /* unknown baud rate */
#define  COM_COB_ID_SYNC        66 /* COB-ID SYNC out of range */
#define  COM_SYNC_RELOAD        67 /* sync timer reload value out of range */
#define  COM_I_OFFSET_MAX       68 /* input offset of a PDO too large */
#define  COM_O_OFFSET_MAX       69 /* output offset of a PDO too large */
#define  COM_PDO_ADD_TAB_INCONS 70 /* inconsistence between PDO and Add tab */
#define  COM_ADD_TAB_INCON      71 /* length of add tab inconsistent */
#define  COM_DATA_LENGTH_INCON  72 /* the whole data length is inconsistent */
#define  COM_COB_ID_EMCY        73 /* COB-ID Emergency out of range */
#define  COM_COB_ID_ND_GUARD    74 /* COB-ID for node guard out of range */
#define  COM_PDO_LENGTH         75 /* configured PDO length larger than 8 */
#define  COM_SDO_DATA_SIZE      76 /* to many objects defined in SDO data */


#if defined( _MSC_VER)          /* Microsoft C */
    #pragma pack()              /* Byte Alignment   */
#endif

#ifdef __cplusplus
  }
#endif /* __cplusplus */

#endif

/* === eof 'COM_USER.H' === */
